@@ -27,8 +27,8 @@
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 __productname__ = "nusex"
-__version__ = "1.0.2"
-__description__ = "A project templating utility for Python."
+__version__ = "1.2.2"
+__description__ = "A dynamic, multi-language project templating utility."
 __url__ = "https://github.com/nusex/nusex"
 __docs__ = "https://nusex.readthedocs.io/en/latest"
 __author__ = "Ethan Henderson"
