@@ -28,6 +28,7 @@
 
 import argparse
 import datetime as dt
+import logging
 import sys
 import traceback
 from importlib import import_module
@@ -53,6 +54,11 @@ COMMAND_MAPPING = {
 parser = argparse.ArgumentParser(description=__description__)
 parser.add_argument(
     "-v",
+    "--verbose",
+    help="emit logging messages",
+    action="store_true",
+)
+parser.add_argument(
     "--version",
     help="show nusex's version and exit",
     action="store_true",
@@ -126,22 +132,19 @@ def _check_for_updates():
 def main():
     args = parser.parse_args()
 
-    # NOTE: Remove in v1.1.
-    if parser.prog == "nsx":
-        cprint(
-            "war",
-            (
-                "The 'nsx' command is deprecated, and will be removed in v1.1 "
-                "(use 'nusex' instead)."
-            ),
-        )
-
     if args.version:
         return print(__version__)
 
     if not args.subparser:
         return parser.parse_args(("-h",))
 
+    if args.verbose:
+        logging.basicConfig(
+            level=logging.DEBUG,
+            format="[%(levelname)s] %(name)s: %(message)s",
+            datefmt="%Y-%m-%d %H:%M:%S",
+        )
+
     # # Setup checks.
     _check_config(args.subparser)
     _check_init(args.subparser)
@@ -153,7 +156,7 @@ def main():
             **{
                 k: v
                 for k, v in args.__dict__.items()
-                if k not in ("subparser", "version")
+                if k not in ("subparser", "verbose", "version")
             }
         )
     except NusexUserError as exc:
@@ -162,9 +165,9 @@ def main():
     except NusexError as exc:
         cprint("err", f"{exc}.")
         sys.exit(1)
-    except KeyboardInterrupt as exc:
+    except KeyboardInterrupt:
         sys.exit(130)
-    except Exception as exc:
+    except Exception:
         cprint(
             "err",
             f"Oh no! Something went wrong.\n\n{traceback.format_exc()}",
