@@ -26,29 +26,14 @@
 # OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
+import logging
 import os
 
-from nusex import TEMPLATE_DIR, Template
-from nusex.errors import AlreadyExists
-from nusex.helpers import cprint
+from nusex import BLUEPRINT_MAPPING, TEMPLATE_DIR, Template
+from nusex.errors import AlreadyExists, DoesNotExist
+from nusex.helpers import cprint, options_as_list, options_as_set
 
-
-def _options_as_set(values):
-    s = set(values.split(","))
-
-    if s == {""}:
-        return {}
-
-    return s
-
-
-def _options_as_list(values):
-    l = values.split(",")
-
-    if l == [""]:
-        return []
-
-    return l
+log = logging.getLogger(__name__)
 
 
 def _check(template):
@@ -81,6 +66,9 @@ def run(
     overwrite,
     check,
     from_repo,
+    project_name,
+    language,
+    as_addon_for,
     with_installs,
     with_requirements_file,
     ignore_exts,
@@ -88,6 +76,25 @@ def run(
     ignore_dirs,
     extend_ignore_dirs,
 ):
+    log.debug(
+        (
+            f"Using CLI values: "
+            f"{name=}; "
+            f"{overwrite=}; "
+            f"{check=}; "
+            f"{from_repo=}; "
+            f"{project_name=}; "
+            f"{language=}; "
+            f"{as_addon_for=}; "
+            f"{with_installs=}; "
+            f"{with_requirements_file=}; "
+            f"{ignore_exts=}; "
+            f"{extend_ignore_exts=}; "
+            f"{ignore_dirs=}; "
+            f"{extend_ignore_dirs=}"
+        )
+    )
+
     if os.path.isfile(TEMPLATE_DIR / f"{name}.nsx") and not overwrite:
         raise AlreadyExists(
             "That template already exists (use -o to overwrite)"
@@ -102,20 +109,35 @@ def run(
             d.remove("")
             with_installs.extend(d)
 
-    # TODO: Make so when overwriting a template, it doesn't have to
-    # load the previous one first.
+    if language not in BLUEPRINT_MAPPING.keys():
+        raise DoesNotExist("That language is not supported")
+    blueprint = BLUEPRINT_MAPPING[language]
+
+    if as_addon_for and not os.path.isfile(
+        TEMPLATE_DIR / f"{as_addon_for}.nsx"
+    ):
+        raise DoesNotExist(
+            "You cannot build an add-on for a template that does not exist"
+        )
+
     if from_repo:
         template = Template.from_repo(
             name,
             from_repo,
+            project_name=project_name,
+            blueprint=blueprint,
             installs=with_installs,
+            as_addon_for=as_addon_for,
             ignore_exts=ignore_exts,
             ignore_dirs=ignore_dirs,
         )
     else:
         template = Template.from_cwd(
             name,
+            project_name=project_name,
+            blueprint=blueprint,
             installs=with_installs,
+            as_addon_for=as_addon_for,
             ignore_exts=ignore_exts,
             ignore_dirs=ignore_dirs,
         )
@@ -154,6 +176,30 @@ def setup(subparsers):
         ),
         metavar="URL",
         default="",
+        type=lambda x: x or None,
+    )
+    s.add_argument(
+        "-p",
+        "--project-name",
+        help="the project name to use (default: name of parent directory)",
+        metavar="NAME",
+        default="",
+        type=lambda x: x or None,
+    )
+    s.add_argument(
+        "-l",
+        "--language",
+        help="the language to assume the project is using (default: python)",
+        metavar="LANGUAGE",
+        default="python",
+        type=lambda x: x.lower(),
+    )
+    s.add_argument(
+        "-a",
+        "--as-addon-for",
+        help="the template this template should be an add-on for",
+        metavar="TEMPLATE",
+        default="",
     )
     s.add_argument(
         "-i",
@@ -163,7 +209,7 @@ def setup(subparsers):
         ),
         metavar="DEPS",
         default="",
-        type=_options_as_list,
+        type=options_as_list,
     )
     s.add_argument(
         "-I",
@@ -183,7 +229,7 @@ def setup(subparsers):
         ),
         metavar="EXTS",
         default="pyc,pyd,pyo",
-        type=_options_as_set,
+        type=options_as_set,
     )
     s.add_argument(
         "--extend-ignore-exts",
@@ -193,7 +239,7 @@ def setup(subparsers):
         ),
         metavar="EXTS",
         default="",
-        type=_options_as_set,
+        type=options_as_set,
     )
     s.add_argument(
         "--ignore-dirs",
@@ -201,14 +247,16 @@ def setup(subparsers):
             "a comma-separated list of directories to ignore; look at the "
             "docs for more information about advanced ignoring syntaxes "
             "(default: .direnv,.eggs,.git,.hg,.mypy_cache,.nox,.tox,.venv,"
-            "venv,.svn,_build,build,dist,buck-out,.pytest-cache,*.egg-info)"
+            "venv,.svn,_build,build,dist,buck-out,.pytest_cache,.coverage,"
+            ".nusexmeta,*.egg-info)"
         ),
         metavar="DIRS",
         default=(
             ".direnv,.eggs,.git,.hg,.mypy_cache,.nox,.tox,.venv,venv,.svn,"
-            "_build,build,dist,buck-out,.pytest-cache,*.egg-info"
+            "_build,build,dist,buck-out,.pytest_cache,.coverage,.nusexmeta,"
+            "*.egg-info"
         ),
-        type=_options_as_set,
+        type=options_as_set,
     )
     s.add_argument(
         "--extend-ignore-dirs",
@@ -218,6 +266,6 @@ def setup(subparsers):
         ),
         metavar="DIRS",
         default="",
-        type=_options_as_set,
+        type=options_as_set,
     )
     return subparsers
