@@ -27,6 +27,7 @@
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 import json
+import logging
 import os
 
 from nusex import CONFIG_DIR, LICENSE_DIR, PROFILE_DIR, VERSION_PATTERN
@@ -43,6 +44,8 @@ VALID_CONFIG_KEYS = (
     "preferred_license",
 )
 
+log = logging.getLogger(__name__)
+
 
 class Profile:
     """A class in which to create, load, modify, and save profiles.
@@ -64,8 +67,10 @@ class Profile:
         self.path = PROFILE_DIR / f"{name}.nsp"
 
         if not os.path.isfile(self.path):
+            log.info(f"[{name}] Not found; creating new...")
             return self.create_new(name)
 
+        log.info(f"[{name}] Loading data...")
         self.load()
 
     def __str__(self):
@@ -112,14 +117,8 @@ class Profile:
             :obj:`AlreadyExists`: The profile already exists on disk.
         """
         validate_name(name, self.__class__.__name__)
-        self.data = {
-            "author_name": "",
-            "author_email": "",
-            "git_profile_url": "",
-            "starting_version": "0.1.0",
-            "default_description": "My project, created using nusex",
-            "preferred_license": "unlicense",
-        }
+        self.data = NSPSpecIO().defaults
+        log.debug(f"[{self.name}] Data = {self.data}")
 
     def load(self):
         """Load an existing profile. This should never need to be called
@@ -131,6 +130,7 @@ class Profile:
                 disk.
         """
         self.data = NSPSpecIO().read(self.path)
+        log.debug(f"[{self.name}] Data = {self.data}")
 
     def save(self):
         """Save this profile.
@@ -139,12 +139,8 @@ class Profile:
             :obj:`ProfileError`: The profile data has been improperly
                 modified.
         """
-        try:
-            NSPSpecIO().write(self.path, self.data)
-        except KeyError:
-            raise ProfileError(
-                "The profile data has been improperly modified"
-            ) from None
+        NSPSpecIO().write(self.path, self.data)
+        log.info(f"[{self.name}] Saved to {self.path}")
 
     def delete(self):
         """Delete this profile.
@@ -154,6 +150,7 @@ class Profile:
                 disk.
         """
         os.remove(self.path)
+        log.info(f"[{self.name}] Deleted from {self.path}")
 
     def rename(self, new_name):
         """Rename this profile.
@@ -162,6 +159,10 @@ class Profile:
             new_name (:obj:`str`): The new name for the profile.
 
         Raises:
+            :obj:`ProfileError`: The new name for the profile is
+                invalid.
+            :obj:`AlreadyExists`: The new name for the profile is
+                already being used by a template.
             :obj:`FileNotFoundError`: The profile does not exist on
                 disk.
         """
@@ -169,6 +170,7 @@ class Profile:
         new_path = f"{self.path}".replace(self.path.stem, new_name)
         self.path.rename(new_path)
         self.path = PROFILE_DIR / f"{new_name}.nsp"
+        log.info(f"[{self.name}] Renamed")
 
     @classmethod
     def current(cls):
@@ -196,6 +198,7 @@ class Profile:
         """
         with open(CONFIG_DIR / "user.nsc") as f:
             data = json.load(f)
+            log.debug(f"[{name}] Old config data: {data}")
 
         c = cls(name)
         values = list(data.values())
@@ -211,6 +214,7 @@ class Profile:
                         or "0.1.0"
                     )
                     v = c._validate_option(k, v)
+                    log.info(f"[{name}] Option '{k}' resolved to '{v}'")
 
             elif k == "preferred_license":
                 v = c._resolve_license(v)
@@ -220,8 +224,10 @@ class Profile:
                         or "unlicense"
                     )
                     v = c._validate_option(k, v)
+                    log.info(f"[{name}] Option '{k}' resolved to '{v}'")
 
             c.data[k] = v
+            log.debug(f"[{name}] Option '{k}' set to '{v}' in profile data")
 
         return c
 
@@ -235,14 +241,19 @@ class Profile:
         return NSCSpecIO().read()["profile"] == self.path.stem
 
     def select(self):
-        """Select this profile."""
+        """Select this profile. This will not error if the profile is
+        already selected; use the :obj:`is_selected` property to check
+        if this profile is already selected instead.
+        """
         data = NSCSpecIO().read()
         data["profile"] = self.path.stem
         NSCSpecIO().write(data)
+        log.info(f"[{self.name}] Selected")
 
     def _resolve_license(self, value):
         for file in LICENSE_DIR.glob("*.txt"):
             if value == file.stem:
+                log.debug(f"[{self.name}] License found: {value}")
                 return value
 
             attrs = {}
@@ -271,6 +282,7 @@ class Profile:
                 )
                 == value
             ):
+                log.debug(f"[{self.name}] License attrs ({value}): {attrs}")
                 return file.stem
 
     def _validate_option(self, key, option):
@@ -291,6 +303,7 @@ class Profile:
                     "Your input could not be resolved to a valid license"
                 )
 
+        log.debug(f"[{self.name}] Option '{key}' resolved to '{option}'")
         return option
 
     def setup(self):
@@ -331,3 +344,4 @@ class Profile:
             if v:
                 v = self._validate_option(k, v)
                 self.data[k] = v
+                log.debug(f"[{self.name}] Option '{k}' updated to '{v}'")
