@@ -28,6 +28,7 @@
 
 import datetime as dt
 import json
+import logging
 import os
 import sys
 from pathlib import Path
@@ -36,8 +37,8 @@ from platform import python_implementation
 from nusex import TEMP_DIR, TEMPLATE_DIR, Profile
 from nusex.blueprints import PythonBlueprint
 from nusex.constants import LICENSE_DIR
-from nusex.errors import BuildError, IncompatibilityError, TemplateError
-from nusex.helpers import run, validate_name
+from nusex.errors import BuildError, IncompatibilityError
+from nusex.helpers import cprint, run, validate_name
 from nusex.spec import NSXSpecIO
 
 ATTRS = (
@@ -53,6 +54,8 @@ ATTRS = (
     "PROJECTBASEEXC",
 )
 
+log = logging.getLogger(__name__)
+
 
 class Template:
     """A class in which to create, load, modify, and save templates.
@@ -64,23 +67,32 @@ class Template:
 
     Keyword Args:
         installs (:obj:`list[str]`): A list of dependancies to be
-            installed when the template is deployed.
+            installed when the template is deployed. Defaults to an
+            empty list.
+        as_addon_for (:obj:`str`): The name of the template this
+            template is an add-on for. Defaults to an empty string.
 
     Attributes:
         path (:obj:`pathlib.Path`): The complete filepath to the
             template.
         data (:obj:`dict[str, Any]`): The data for the template.
+
+    .. versionchanged:: 1.2
+            Added ``as_addon_for`` keyword argument.
     """
 
-    __slots__ = ("path", "data", "_installs")
+    __slots__ = ("path", "data", "_installs", "_as_addon_for")
 
-    def __init__(self, name, *, installs=[]):
+    def __init__(self, name, *, installs=[], as_addon_for=""):
         self.path = TEMPLATE_DIR / f"{name}.nsx"
         self._installs = installs
+        self._as_addon_for = as_addon_for
 
-        if not os.path.isfile(self.path):
+        if not self.path.exists():
+            log.info(f"[{name}] Not found; creating new...")
             return self.create_new(name)
 
+        log.info(f"[{name}] Loading data...")
         self.load()
 
     def __str__(self):
@@ -131,11 +143,8 @@ class Template:
             :obj:`AlreadyExists`: The template already exists on disk.
         """
         validate_name(name, self.__class__.__name__)
-        self.data = {
-            "files": {},
-            "installs": [],
-            "as_extension_for": "",
-        }
+        self.data = NSXSpecIO().defaults
+        log.debug(f"[{self.name}] Data = {self.data}")
 
     def load(self):
         """Load an existing template. This should never need to be
@@ -147,6 +156,7 @@ class Template:
                 disk.
         """
         self.data = NSXSpecIO().read(self.path)
+        log.debug(f"[{self.name}] Files = {list(self.data['files'].keys())}")
 
     def save(self):
         """Save this profile.
@@ -155,12 +165,8 @@ class Template:
             :obj:`TemplateError`: The profile data has been improperly
                 modified.
         """
-        try:
-            NSXSpecIO().write(self.path, self.data)
-        except KeyError:
-            raise TemplateError(
-                "The template data has been improperly modified"
-            ) from None
+        NSXSpecIO().write(self.path, self.data)
+        log.info(f"[{self.name}] Saved to {self.path}")
 
     def delete(self):
         """Delete this template.
@@ -170,6 +176,7 @@ class Template:
                 disk.
         """
         os.remove(self.path)
+        log.info(f"[{self.name}] Deleted from {self.path}")
 
     def rename(self, new_name):
         """Rename this template.
@@ -178,6 +185,10 @@ class Template:
             new_name (:obj:`str`): The new name for the template.
 
         Raises:
+            :obj:`TemplateError`: The new name for the template is
+                invalid.
+            :obj:`AlreadyExists`: The new name for the template is
+                already being used by a profile.
             :obj:`FileNotFoundError`: The template does not exist on
                 disk.
         """
@@ -185,10 +196,19 @@ class Template:
         new_path = f"{self.path}".replace(self.path.stem, new_name)
         self.path.rename(new_path)
         self.path = TEMPLATE_DIR / f"{new_name}.nsx"
+        log.info(f"[{self.name}] Renamed")
 
     @classmethod
     def from_cwd(
-        cls, name, *, installs=[], ignore_exts=set(), ignore_dirs=set()
+        cls,
+        name,
+        *,
+        project_name=None,
+        blueprint=None,
+        installs=[],
+        as_addon_for="",
+        ignore_exts=set(),
+        ignore_dirs=set(),
     ):
         """Create a template using files from the current working
         directory.
@@ -197,9 +217,17 @@ class Template:
             name (:obj:`str`): The name of the template.
 
         Keyword Args:
+            project_name (:obj:`str`): The name to use as the project
+                name when building the project. If this is None, the
+                name of the parent directory is used. Defaults to None.
+            blueprint (:obj:`Blueprint`): The language blueprint to use.
+                If this is None, the Python blueprint is selected.
+                Defaults to None.
             installs (:obj:`list[str]`): A list of dependencies to
                 install when this template is deployed. Defaults to an
                 empty list.
+            as_addon_for (:obj:`str`): The name of the template this
+                template is an add-on for. Defaults to an empty string.
             ignore_exts (:obj:`set[str]`): A set of file extensions to
                 ignore. Defaults to an empty set.
             ignore_dirs (:obj:`set[str]`): A set of directories to
@@ -207,9 +235,17 @@ class Template:
 
         Returns:
             :obj:`Template`: The newly created template.
+
+        .. versionchanged:: 1.1
+            Added ``project_name`` and ``blueprint`` keyword arguments.
+
+        .. versionchanged:: 1.2
+            Added ``as_addon_for`` keyword argument.
         """
-        c = cls(name, installs=installs)
+        c = cls(name, installs=installs, as_addon_for=as_addon_for)
         c.build(
+            project_name=project_name,
+            blueprint=blueprint,
             ignore_exts=ignore_exts,
             ignore_dirs=ignore_dirs,
         )
@@ -217,7 +253,16 @@ class Template:
 
     @classmethod
     def from_dir(
-        cls, name, path, *, installs=[], ignore_exts=set(), ignore_dirs=set()
+        cls,
+        name,
+        path,
+        *,
+        project_name=None,
+        blueprint=None,
+        installs=[],
+        as_addon_for="",
+        ignore_exts=set(),
+        ignore_dirs=set(),
     ):
         """Create a template using files from a specific directory.
 
@@ -227,9 +272,17 @@ class Template:
                 files to build the template with.
 
         Keyword Args:
+            project_name (:obj:`str`): The name to use as the project
+                name when building the project. If this is None, the
+                name of the parent directory is used. Defaults to None.
+            blueprint (:obj:`Blueprint`): The language blueprint to use.
+                If this is None, the Python blueprint is selected.
+                Defaults to None.
             installs (:obj:`list[str]`): A list of dependencies to
                 install when this template is deployed. Defaults to an
                 empty list.
+            as_addon_for (:obj:`str`): The name of the template this
+                template is an add-on for. Defaults to an empty string.
             ignore_exts (:obj:`set[str]`): A set of file extensions to
                 ignore. Defaults to an empty set.
             ignore_dirs (:obj:`set[str]`): A set of directories to
@@ -237,10 +290,18 @@ class Template:
 
         Returns:
             :obj:`Template`: The newly created template.
+
+        .. versionchanged:: 1.1
+            Added ``project_name`` and ``blueprint`` keyword arguments.
+
+        .. versionchanged:: 1.2
+            Added ``as_addon_for`` keyword argument.
         """
-        c = cls(name, installs=installs)
+        c = cls(name, installs=installs, as_addon_for=as_addon_for)
         c.build(
+            project_name=project_name,
             root_dir=path,
+            blueprint=blueprint,
             ignore_exts=ignore_exts,
             ignore_dirs=ignore_dirs,
         )
@@ -248,7 +309,16 @@ class Template:
 
     @classmethod
     def from_repo(
-        cls, name, url, *, installs=[], ignore_exts=set(), ignore_dirs=set()
+        cls,
+        name,
+        url,
+        *,
+        project_name=None,
+        blueprint=None,
+        installs=[],
+        as_addon_for="",
+        ignore_exts=set(),
+        ignore_dirs=set(),
     ):
         """Create a template using files from a GitHub repository.
 
@@ -257,9 +327,17 @@ class Template:
             url (:obj:`str`): The URL of the GitHub repository to clone.
 
         Keyword Args:
+            project_name (:obj:`str`): The name to use as the project
+                name when building the project. If this is None, the
+                name of the parent directory is used. Defaults to None.
+            blueprint (:obj:`Blueprint`): The language blueprint to use.
+                If this is None, the Python blueprint is selected.
+                Defaults to None.
             installs (:obj:`list[str]`): A list of dependencies to
                 install when this template is deployed. Defaults to an
                 empty list.
+            as_addon_for (:obj:`str`): The name of the template this
+                template is an add-on for. Defaults to an empty string.
             ignore_exts (:obj:`set[str]`): A set of file extensions to
                 ignore. Defaults to an empty set.
             ignore_dirs (:obj:`set[str]`): A set of directories to
@@ -270,21 +348,34 @@ class Template:
 
         Raises:
             :obj:`BuildError`: Cloning the repository failed.
+
+        .. versionchanged:: 1.1
+            Added ``project_name`` and ``blueprint`` keyword arguments.
+
+        .. versionchanged:: 1.2
+            Added ``as_addon_for`` keyword argument.
         """
         os.makedirs(TEMP_DIR, exist_ok=True)
         os.chdir(TEMP_DIR)
 
-        output = run(f"git clone {url}")
-        if output.returncode == 1:
+        log.debug(f"[{name}] Cloning {url} into {TEMP_DIR}...")
+        run(f"git clone {url}")
+
+        repo_dir = TEMP_DIR / url.split("/")[-1].replace(".git", "")
+        if not repo_dir.exists():
             raise BuildError(
                 "Cloning the repo failed. Is Git installed? Is the URL "
                 "correct?"
             )
 
-        os.chdir(TEMP_DIR / url.split("/")[-1].replace(".git", ""))
+        os.chdir(repo_dir)
+        log.debug(f"[{name}] Building from {os.path.abspath(os.curdir)}...")
         return cls.from_cwd(
             name,
+            project_name=project_name,
+            blueprint=blueprint,
             installs=installs,
+            as_addon_for=as_addon_for,
             ignore_exts=ignore_exts,
             ignore_dirs=ignore_dirs,
         )
@@ -320,25 +411,43 @@ class Template:
             filter(lambda x: x.startswith("*"), ignore_dirs)
         )
         true_dir_ignores = ignore_dirs - wild_dir_ignores
+        log.debug(f"[{self.name}] Ignoring exts: {ignore_exts}")
+        log.debug(f"[{self.name}] Ignoring dirs (true): {true_dir_ignores}")
+        log.debug(f"[{self.name}] Ignoring dirs (wild): {wild_dir_ignores}")
+
         files = filter(lambda p: is_valid(p), Path(root_dir).rglob("*"))
         return list(files)
 
-    def build(self, project_name=None, files=[], root_dir=".", **kwargs):
+    def build(
+        self,
+        *,
+        project_name=None,
+        files=[],
+        root_dir=".",
+        blueprint=None,
+        **kwargs,
+    ):
         """Build this template. View the
         :doc:`template guide <../guide/templates>` to see what this
         command does in more detail.
 
         Keyword Args:
-            project_name (:obj:`str`): The name of the project. If this
-                is None the project name is set to the name of the
-                parent folder. Defaults to None.
+            project_name (:obj:`str`): The name to use as the project
+                name when building the project. If this is None, the
+                name of the parent directory is used. Defaults to None.
             files (:obj:`list[str]`): The list of files to include in
                 this template. If no files are specified, the file
                 listing is automatically retrieved.
             root_dir (:obj:`str`): The root directory that nusex will
                 search from. Defaults to the current directory.
+            blueprint (:obj:`Blueprint`): The language blueprint to use.
+                If this is None, the Python blueprint is selected.
+                Defaults to None.
             **kwargs (:obj:`Any`): Arguments for the
                 :obj:`get_file_listing` method.
+
+        .. versionchanged:: 1.1
+            Added ``blueprint`` keyword argument.
         """
 
         def resolve_key(path):
@@ -348,6 +457,9 @@ class Template:
         if not project_name:
             project_name = Path(root_dir).resolve().parts[-1]
 
+        if not blueprint:
+            blueprint = PythonBlueprint
+
         if not files:
             files = self.get_file_listing(
                 root_dir,
@@ -355,29 +467,43 @@ class Template:
                 ignore_dirs=kwargs.pop("ignore_dirs", set()),
             )
 
+        self.data["installs"] = self._installs
+        self.data["as_addon_for"] = self._as_addon_for
+
+        log.info(f"[{self.name}] Using project name: {project_name}")
+        log.info(f"[{self.name}] Using blueprint: {blueprint.__name__}")
+        log.info(f"[{self.name}] As add-on for: " + self.data["as_addon_for"])
+        log.info(f"[{self.name}] For language: " + self.data["language"])
+        log.info(f"[{self.name}] With {len(files):,} files")
+        log.debug(f"[{self.name}] With files: {files}")
+
         nparts = len(Path(root_dir).resolve().parts)
-        data = {
-            "files": {resolve_key(f): f.read_bytes() for f in files},
-            "installs": self._installs,
-            "as_extension_for": "",
-        }
+        self.data["files"] = {resolve_key(f): f.read_bytes() for f in files}
 
-        blueprint = PythonBlueprint(project_name, data)
-        self.data = blueprint().data
+        bp = blueprint(project_name, self.data)
+        self.data = bp().data
 
-    def deploy(self, path="."):
+        log.info(f"[{self.name}] Build successful")
+
+    def deploy(self, *, project_name=None, destination="."):
         """Deploy this template.
 
         Keyword Args:
-            path (:obj:`str`): The path to deploy this template to.
-                Defaults to the current directory.
+            project_name (:obj:`str`): The name to use as the project
+                name when deploying the project. If this is None, the
+                name of the parent directory is used. Defaults to None.
+            destination (:obj:`str`): The path to deploy this template
+                to. Defaults to the current directory.
+
+        .. versionchanged:: 1.1
+            Added ``project_name`` keyword argument.
         """
 
         def resolve_version(key):
-            if key != "CALVER":
-                return key
+            if key == "CALVER":
+                return dt.date.today().strftime("%Y.%m.%d")
 
-            return dt.date.today().strftime("%Y.%m.%d")
+            return key
 
         def resolve_license_info(key):
             with open(LICENSE_DIR / f"{key}.txt", encoding="utf-8") as f:
@@ -388,22 +514,25 @@ class Template:
             ] + 2
 
             return (
-                lines[1][7:],
+                lines[1][7:].replace('"', "'"),
                 "\n".join(lines[start:])
                 .replace("[year]", f"{dt.date.today().year}")
                 .replace("[fullname]", profile["author_name"]),
             )
 
-        profile = Profile.current()
-        project_name = Path(path).resolve().parts[-1]
-        lic_name, lic_body = resolve_license_info(profile["preferred_license"])
+        if not project_name:
+            project_name = Path(destination).resolve().parts[-1]
+        project_slug = project_name.lower().replace(" ", "_").replace("-", "_")
         project_error = project_name.replace("_", " ").title().replace(" ", "")
 
+        profile = Profile.current()
+        lic_name, lic_body = resolve_license_info(profile["preferred_license"])
+
         var_mapping = {
             b"PROJECTNAME": project_name,
             b"PROJECTVERSION": resolve_version(profile["starting_version"]),
             b"PROJECTDESCRIPTION": profile["default_description"],
-            b"PROJECTURL": f"{profile['git_profile_url']}/{project_name}",
+            b"PROJECTURL": f"{profile['git_profile_url']}/{project_slug}",
             b"PROJECTAUTHOREMAIL": profile["author_email"],
             b"PROJECTAUTHOR": profile["author_name"],
             b"PROJECTLICENSE": lic_name,
@@ -412,28 +541,33 @@ class Template:
             b"PROJECTBASEEXC": f"{project_error}Error",
         }
 
+        log.info(f"[{self.name}] Using project name: {project_name}")
+        log.info(f"[{self.name}] Using project slug: {project_slug}")
+        log.debug(f"[{self.name}] Using var mapping: {var_mapping}")
+
         for name, data in self.data["files"].items():
-            name = name.replace(
-                "PROJECTNAME",
-                project_name.lower().replace(" ", "_").replace("-", "_"),
-            )
+            name = name.replace("PROJECTNAME", project_slug)
 
             dirs = name.split("/")[:-1]
             if dirs:
-                os.makedirs(f"{path}/" + "/".join(dirs), exist_ok=True)
+                os.makedirs(f"{destination}/" + "/".join(dirs), exist_ok=True)
 
             for k, v in var_mapping.items():
                 data = data.replace(k, v.encode())
 
-            with open(f"{path}/{name}", "wb") as f:
+            with open(f"{destination}/{name}", "wb") as f:
                 f.write(data)
 
         meta = {
             "template": self.name,
             "files": list(self.data["files"].keys()),
+            "language": self.data["language"],
         }
-        with open(f"{path}/.nusexmeta", "w") as f:
-            json.dump(meta, f)
+        if not self.data["as_addon_for"]:
+            with open(f"{destination}/.nusexmeta", "w") as f:
+                json.dump(meta, f)
+
+        log.info(f"[{self.name}] Deployment successful")
 
     def install_dependencies(self):
         """Install this template's dependencies. Note that this does not
@@ -444,13 +578,22 @@ class Template:
                 "implementations"
             )
 
-        if not self.data["installs"]:
+        if self.data["language"] != "python":
+            cprint(
+                "war",
+                "Dependency installation is not supported on languages other "
+                "than Python",
+            )
             return
 
-        run(
-            f"{sys.executable} -m pip install "
-            + " ".join(self.data["installs"])
-        )
+        installs = self.data["installs"]
+
+        if not installs:
+            log.info(f"[{self.name}] No dependencies to install")
+            return
+
+        log.info(f"[{self.name}] Installing {len(installs):,} dependencies...")
+        run(f"{sys.executable} -m pip install " + " ".join(installs))
 
     def check(self):
         """Check the template manifest, including line changes.
